#include <config.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gui.h>
#include <stdlib.h>
#include <string.h>
#include <plugin.h>
#include <stdio.h>
#include <node.h>
#include <floor.h>
#include <loop.h>
#include <guisupport.h>

extern GList *floorList;
static GtkWidget *mainWindow;

static GtkWidget *lightSwitch;
static GtkWidget *toolBar;

Floor *currentFloor;

GtkItemFactoryEntry menuItems[]={
    {"/_File", NULL, 0, 0, "<Branch>"},
    {"/File/_New" , "<control>N", GTK_SIGNAL_FUNC(new_menu_activate) , 0, NULL},
    {"/File/_Open", "<control>O", GTK_SIGNAL_FUNC(open_menu_activate), 0, NULL},
    {"/File/_Save", "<control>S", GTK_SIGNAL_FUNC(save_menu_activate), 0, NULL},
    {"/File/Save _As", "<control>A", GTK_SIGNAL_FUNC(saveas_menu_activate), 0, NULL},
    {"/File/sep1",  NULL , NULL, 0, "<Separator>"},
    {"/File/_Quit", "<alt>Q"    , GTK_SIGNAL_FUNC(quit_menu_activate), 0, NULL},

    {"/_Options", NULL, 0, 0, "<Branch>"},
    {"/Options/_Settings","<control>G",GTK_SIGNAL_FUNC(settings_menu_activate),0,NULL},

    {"/_Help", NULL, 0, 0, "<LastBranch>"},
    {"/Help/_Using Stompboxes2", NULL, GTK_SIGNAL_FUNC(show_quickstart_help), 0, NULL},
    {"/Help/_About", NULL, GTK_SIGNAL_FUNC(about_menu_activate), 0, NULL}
};

Floor *get_current_floor(){
    //return (Floor *)floorList->data;
    return currentFloor;
}

int set_current_floor(void *floor){
    if(g_list_index(floorList,floor)==-1){
	floorList = g_list_append(floorList,(Floor *)floor);
    }
    currentFloor = (Floor *)floor;
    return TRUE;
}

int main_window_destroy(GtkWidget *widget, gpointer data){
    gtk_widget_destroy(widget);
    gtk_main_quit();
    return 0;
}

int quit_menu_activate(GtkWidget *widget, gpointer data){
    gtk_widget_destroy(mainWindow);
    return TRUE;
}


int new_menu_activate(GtkWidget *widget, gpointer data){
    Floor *floor = gui_floor_new();
    floor->drawingArea = currentFloor->drawingArea;
    floor->pixmap = currentFloor->pixmap;
    floor->bgColor = currentFloor->bgColor;
    floor->linksColor = currentFloor->linksColor;
    floor->nodesColor = currentFloor->nodesColor;
    floor->fontColor = currentFloor->fontColor;
    floor->blackColor = currentFloor->blackColor;
    floor->whiteColor = currentFloor->whiteColor;

    set_current_floor(floor);
    floor_repaint(floor);
    return TRUE;
}

void gtk_widget_destroy_callback(GtkWidget *widget, gpointer data){
    gtk_widget_destroy(data);
}

int open_menu_callback(GtkFileSelection *widget, gpointer data){
    Floor *floor = get_current_floor();
    gchar *fileName;
    fileName = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));
    floor_open_with_name(floor,fileName);
    return TRUE;    
}

int open_menu_activate(GtkWidget *widget, gpointer data){
    GtkWidget *fileSelector;
    fileSelector = gtk_file_selection_new("Open");
    gtk_signal_connect(GTK_OBJECT(
		(GTK_FILE_SELECTION(fileSelector)->ok_button)),
	    "clicked", GTK_SIGNAL_FUNC (open_menu_callback), fileSelector);
    gtk_signal_connect(GTK_OBJECT((GTK_FILE_SELECTION(fileSelector)->ok_button)),
	    "clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy_callback),
	    (gpointer)fileSelector);
    gtk_signal_connect(GTK_OBJECT((GTK_FILE_SELECTION(fileSelector)->cancel_button)),
	    "clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy_callback),
	    (gpointer)fileSelector);
    gtk_widget_show(fileSelector);

    return TRUE;
}

int save_menu_callback(GtkFileSelection *widget, gpointer data){
    Floor *floor = get_current_floor();
    gchar *fileName;
    fileName = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));
    floor_save_with_name(floor,fileName);
    return TRUE;    
}
int save_menu_activate(GtkWidget *widget, gpointer data){
    Floor *floor = get_current_floor();
    GtkWidget *fileSelector;
    if(strlen(floor->floorFileName)==0){
	fileSelector = gtk_file_selection_new("Save As");
	gtk_signal_connect(GTK_OBJECT(
		    (GTK_FILE_SELECTION(fileSelector)->ok_button)),
		"clicked", GTK_SIGNAL_FUNC (save_menu_callback), fileSelector);
	gtk_signal_connect(GTK_OBJECT((GTK_FILE_SELECTION(fileSelector)->ok_button)),
		"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy_callback),
		(gpointer)fileSelector);
	gtk_signal_connect(GTK_OBJECT((GTK_FILE_SELECTION(fileSelector)->cancel_button)),
		"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy_callback),
		(gpointer)fileSelector);
	gtk_widget_show(fileSelector);
    }
    else{
	floor_save(floor);
    }
    return TRUE;
}

int saveas_menu_activate(GtkWidget *widget, gpointer data){
    GtkWidget *fileSelector;
    fileSelector = gtk_file_selection_new("Save As");
    gtk_signal_connect(GTK_OBJECT(
		(GTK_FILE_SELECTION(fileSelector)->ok_button)),
	    "clicked", GTK_SIGNAL_FUNC (save_menu_callback), fileSelector);
    gtk_signal_connect(GTK_OBJECT((GTK_FILE_SELECTION(fileSelector)->ok_button)),
	    "clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy_callback),
	    (gpointer)fileSelector);
    gtk_signal_connect(GTK_OBJECT((GTK_FILE_SELECTION(fileSelector)->cancel_button)),
	    "clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy_callback),
	    (gpointer)fileSelector);
    gtk_widget_show(fileSelector);
    return TRUE;
}

int about_menu_activate(GtkWidget *widget, gpointer data){
    gchar aboutMessage[256];
    strcpy(aboutMessage,PACKAGE);
    strcat(aboutMessage,"\n\nVersion: ");
    strcat(aboutMessage,VERSION);
    strcat(aboutMessage,"\n\n\n\nHector Urtubia   urtubia@mrbook.org");
    strcat(aboutMessage,"\n\nhttp://mrbook.org/stompboxes");
    create_message_window(aboutMessage);
    return TRUE;
}

int settings_menu_activate(GtkWidget *widget, gpointer data){
    show_settings_dialog();
    return TRUE;   
}

int play_button_clicked(GtkWidget *widget, gpointer data){
    Floor *floor = get_current_floor();
    init_loop_on_floor(floor);
    return TRUE;
}

int stop_button_clicked(GtkWidget *widget, gpointer data){
    stop_loop();
    return TRUE;
}

int add_node_button_clicked(GtkWidget *widget, gpointer data){
    Floor *floor = get_current_floor();
    add_node_menu_activate(widget,floor);
    return TRUE;
}

GtkWidget *get_light_switch_widget(GtkWidget *parent,int state){
    GdkPixmap *pixmap;
    //GtkWidget *pixmapWidget;
    char xpmName[256];
    GtkStyle *style;
    GdkBitmap *mask;
    strcpy(xpmName,PIXMAP_PATH);
    style = gtk_widget_get_style(parent);
    if(state==1){
	strcat(xpmName,"/lighton.xpm");
	pixmap = gdk_pixmap_create_from_xpm (parent->window, &mask,
		&style->bg[GTK_STATE_NORMAL],
		xpmName);
    }
    else{
	strcat(xpmName,"/lightoff.xpm");
	pixmap = gdk_pixmap_create_from_xpm (parent->window, &mask,
		&style->bg[GTK_STATE_NORMAL],
		xpmName);
    }
    return gtk_pixmap_new(pixmap,mask);
}

void update_light_switch(int state){
    GdkBitmap *mask;
    GdkPixmap *pixmap;
    char xpmName[256];
    GtkWidget *parent = mainWindow;
    GtkStyle *style;
    style = gtk_widget_get_style(parent);
    strcpy(xpmName,PIXMAP_PATH);
    gtk_pixmap_get(GTK_PIXMAP(lightSwitch),&pixmap,&mask);
    gdk_pixmap_unref(pixmap);
    if(state==1){
	strcat(xpmName,"/lighton.xpm");
	pixmap = gdk_pixmap_create_from_xpm (parent->window, &mask,
		&style->bg[GTK_STATE_NORMAL],
		xpmName);
    }
    else{
	strcat(xpmName,"/lightoff.xpm");
	pixmap = gdk_pixmap_create_from_xpm (parent->window, &mask,
		&style->bg[GTK_STATE_NORMAL],
		xpmName);
    }
    gtk_pixmap_set(GTK_PIXMAP(lightSwitch),pixmap,mask);
}

void gui_main(int argc,char *argv[]){
    GtkWidget *frame;
    GtkWidget *vbox;
    GtkWidget *menuBar;
    GtkAccelGroup *accelGroup;
    GtkItemFactory *itemFactory;
    gchar window_title[128];

    Floor *stompFloor;

    gtk_init(&argc,&argv);

    mainWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    strcpy(window_title,PACKAGE);
    strcat(window_title,"   version ");
    strcat(window_title,VERSION);
    gtk_window_set_title(GTK_WINDOW(mainWindow),window_title);
    gtk_signal_connect(GTK_OBJECT(mainWindow),"destroy",
	    GTK_SIGNAL_FUNC(main_window_destroy),NULL);
    gtk_widget_realize(GTK_WIDGET(mainWindow));

    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(mainWindow),vbox);

    /* prepare and add the menu */
    accelGroup=gtk_accel_group_new();
    itemFactory=gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accelGroup);
    gtk_item_factory_create_items(itemFactory, 12, menuItems, NULL);
    gtk_accel_group_attach(accelGroup, GTK_OBJECT(mainWindow));
    menuBar = gtk_item_factory_get_widget(itemFactory,"<main>");
    gtk_box_pack_start(GTK_BOX(vbox),menuBar,FALSE,FALSE,0);

    /* prepare and add the toolbar */
    toolBar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
	    GTK_TOOLBAR_TEXT);
    gtk_toolbar_append_space(GTK_TOOLBAR(toolBar));
    gtk_toolbar_append_item(GTK_TOOLBAR(toolBar),"Open","Open a previously saved layout",NULL,
	    NULL,GTK_SIGNAL_FUNC(open_menu_activate),NULL);
    gtk_toolbar_append_item(GTK_TOOLBAR(toolBar),"New","Create a new layout",NULL,
	    NULL,GTK_SIGNAL_FUNC(new_menu_activate),NULL);
    gtk_toolbar_append_item(GTK_TOOLBAR(toolBar),"Save","Save the current layout",NULL,
	    NULL,GTK_SIGNAL_FUNC(save_menu_activate),NULL);
    gtk_toolbar_append_space(GTK_TOOLBAR(toolBar));
    gtk_toolbar_append_item(GTK_TOOLBAR(toolBar),"On","start playing the layout",NULL,
	    NULL,GTK_SIGNAL_FUNC(play_button_clicked),NULL);
    gtk_toolbar_append_item(GTK_TOOLBAR(toolBar),"Off","stop playing the layout",NULL,
	    NULL,GTK_SIGNAL_FUNC(stop_button_clicked),NULL);
    /* code for putting the light switch */
    lightSwitch = get_light_switch_widget(mainWindow,0);
    gtk_toolbar_append_widget(GTK_TOOLBAR(toolBar),lightSwitch,"Light Switch",NULL);

    gtk_toolbar_append_space(GTK_TOOLBAR(toolBar));
    gtk_toolbar_append_space(GTK_TOOLBAR(toolBar));
    gtk_toolbar_append_item(GTK_TOOLBAR(toolBar),"Add StompBox","Add a stompbox to the working floor",NULL,
	    NULL,GTK_SIGNAL_FUNC(add_node_button_clicked),NULL);
    /* now put the toolBar on the vbox */
    gtk_box_pack_start(GTK_BOX(vbox),toolBar,FALSE,FALSE,5);

    frame = gtk_frame_new(NULL);
    //frame = gtk_frame_new("StompBoxes Floor");
    //gtk_frame_set_label_align(GTK_FRAME(frame),.8,.0);

    /* after we get the floor, we should add it to the list
     * of all floors. Extensibililility.
     */
    stompFloor = gui_floor_new();
    currentFloor = stompFloor;
    floorList = g_list_append(floorList,stompFloor);

    gtk_container_add(GTK_CONTAINER(frame),stompFloor->drawingArea);

    gtk_box_pack_start(GTK_BOX(vbox),frame,0,0,FALSE);

    gtk_widget_show_all(mainWindow);

    gtk_main();
}
