#ifndef NODE_H
#define NODE_H
#include <plugin.h>
#include <floor.h>

typedef struct{
    /* the font used to draw */
    GdkFont *textFont;
    /* the name of the shared object */
    gchar so_name[128];
    /* short name of the plugin */
    /* maximum character numbers is 10*/
    gchar name[11];
    /* the current position of the node*/
    GdkPoint position; 
    /* number of inputs and outputs */
    int numInputs;
    int numOutputs;
    /* An array of pointers to nodeConnections
     * for inputs and outputs
     */
    void **inputConnections;
    void **outputConnections;

    /* Plugin data */
    void *pluginData;
    /* pointer to the functions */
    Plugin *pluginInfo;
    
    /* describe the node */
    gint nodeType;

    /* flag to denote if this node
     * has been already been processed
     */
    gboolean processed;
    /* flag to see if is ready to
     * process
     */
    gboolean isReadyForProcessing;
    
    /* This is the buffers that this node is going
     * to use
     */
    short **buffers;
    /* An array with flags that denote the buffers
     * that are ready to process the data
     */
    gboolean *readyBuffers;


}Node;

typedef struct{
    Node *node;
    int port;
}NodeConnection;

#define NODE_TYPE_NORMAL       1<<1
#define NODE_TYPE_MAIN_INPUT   1<<2
#define NODE_TYPE_MAIN_OUTPUT  1<<3
#define NODE_TYPE_CONSUMER     1<<4
#define NODE_TYPE_PRODUCER     1<<5
#define NODE_TYPE_FILTER       1<<6

#define IS_NODE_TYPE_NORMAL(x)      ((x) & NODE_TYPE_NORMAL)
#define IS_NODE_TYPE_MAIN_INPUT(x)  ((x) & NODE_TYPE_MAIN_INPUT)
#define IS_NODE_TYPE_MAIN_OUTPUT(x) ((x) & NODE_TYPE_MAIN_OUTPUT)
#define IS_NODE_TYPE_CONSUMER(x)    ((x) & NODE_TYPE_CONSUMER)
#define IS_NODE_TYPE_PRODUCER(x)    ((x) & NODE_TYPE_PRODUCER)
#define IS_NODE_TYPE_FILTER(x)      ((x) & NODE_TYPE_FILTER)


typedef gint NodeSection;
#define NODE_SECTION_INPUT 1
#define NODE_SECTION_OUTPUT 2
#define NODE_SECTION_BODY 3
#define NODE_SECTION_NONE 4

Node *node_new();
Node *node_main_in();
Node *node_main_out();

void node_destroy(Node *node);
void draw_node_in_floor(Node *node, Floor *floor);
gint point_touches_node(GdkPoint point,Node *node);
gint point_touches_node_section(GdkPoint point,Node *node);
gint node_output_available(Node *node);
gint node_input_available(Node *node);
gint node_connect_input_to_output(Node *inputNode,Node *outputNode);
void node_disconnect_outputs(Node *node);
void node_disconnect_inputs(Node *node);
Node *node_new_with_name(gchar *pluginName);
void node_configure_plugin(Node *node);
Node *get_main_input_node(Floor *floor);
Node *get_main_output_node(Floor *floor);
/* sets the buffers of the output node from the input node
 * according to the input's outputConnection
 */
void node_pass_buffers(Node *inNode, Node *outNode, NodeConnection *outputConnection, int outPort);
void process_node(Node *node, short int **buffers);

#endif
