#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

#include <oss.h>

static int snd_device_handle;
int sample_format;
int stereo;
char sound_device_name[80]="/dev/dsp";

int sound_init_oss(int sampling_frequency, int *blocksize){
    int fragments;
    int caps;

    if((snd_device_handle=open(sound_device_name, O_RDWR)) == -1) return 1;
    ioctl(snd_device_handle, SNDCTL_DSP_SETDUPLEX);

    ioctl(snd_device_handle, SNDCTL_DSP_GETCAPS, &caps);
    if((caps & DSP_CAP_DUPLEX) == 0) return 2;
    if(*blocksize!=0)
    {
	switch(*blocksize)
	{
	    case 64: fragments=0x00000006; break;
	    case 128: fragments=0x00000007; break;
	    case 256: fragments=0x00000008; break;
	    case 1024: fragments=0x0000000A; break;
	    case 2048: fragments=0x0000000B; break;
	    default: fragments=0x00000008; break;
	}
	fragments=fragments | 0x00070000;  /* 8 fragmentow */
    }
    else
    {
	*blocksize=1024;
	fragments=0x0004000A;
    }

    ioctl(snd_device_handle, SNDCTL_DSP_SETFRAGMENT, &fragments);
    /*	ioctl(snd_device_handle, SNDCTL_DSP_RESET);*/
    sample_format=AFMT_S16_LE;
    ioctl(snd_device_handle, SNDCTL_DSP_SETFMT, &sample_format);
    if(ioctl(snd_device_handle, SNDCTL_DSP_SPEED, &sampling_frequency) == -1) return 3;
    stereo=0;
    ioctl(snd_device_handle, SNDCTL_DSP_STEREO, &stereo);

    ioctl(snd_device_handle, SNDCTL_DSP_GETBLKSIZE, blocksize);

    return 0;
}

int sound_stop_oss(void){
    ioctl(snd_device_handle, SNDCTL_DSP_RESET);
    close(snd_device_handle);
    return 1;
}

int sound_write_oss(short int *process_buffer, int blocksize){
    return(write(snd_device_handle, process_buffer, blocksize));
}

int sound_read_oss(short int *process_buffer, int blocksize){
    return(read(snd_device_handle, process_buffer, blocksize));
}

