#include <gtk/gtk.h>
#include <stdlib.h>
#include <plugin.h>
#include <stdio.h>

typedef struct{
    gfloat out1;
    gfloat out2;
    GtkWidget *window;
}pluginData;

void *init(void){
    pluginData *this;
    this = (pluginData *)malloc(sizeof(pluginData));
    this->out1 = 0.5;
    this->out2 = 0.5;
    this->window = NULL;
    return this;
}

static void cleanup(void *data){
    pluginData *this = (pluginData *)data;
    if(this->window!=NULL) gtk_widget_destroy(this->window);
    free(data);
}

static gint window_destroy(GtkWidget *window, gpointer data){
    pluginData *this = (pluginData *)data;
    gtk_widget_destroy(window);
    this->window = NULL;
    return TRUE;
}

static void val1_changed(GtkAdjustment *adj,gpointer data){
    pluginData *this = (pluginData *)data;
    this->out1 = GTK_ADJUSTMENT(adj)->value;
}

static void val2_changed(GtkAdjustment *adj,gpointer data){
    pluginData *this = (pluginData *)data;
    this->out2 = GTK_ADJUSTMENT(adj)->value;
}

static void configure(void *data){
    pluginData *this = (pluginData *)data;
    GtkWidget *hbox,*vbox;
    GtkWidget *label;
    GtkAdjustment *val1,*val2;
    GtkWidget *val1_bar,*val2_bar;
    if(this->window != NULL) return;
    this->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(this->window),"1 to 2 mixer");
    gtk_widget_set_usize(GTK_WIDGET(this->window),300,100);
    gtk_signal_connect(GTK_OBJECT(this->window),"destroy",
	    GTK_SIGNAL_FUNC(window_destroy),data);
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(this->window),vbox);

    hbox = gtk_hbox_new(FALSE,0);
    label = gtk_label_new("out mix 1");
    val1 = (GtkAdjustment *)gtk_adjustment_new(this->out1,0.0,1.0,0.01,0.01,0.0);
    gtk_signal_connect(GTK_OBJECT(val1),"value_changed",GTK_SIGNAL_FUNC(val1_changed),data);
    val1_bar = gtk_hscale_new(val1);
    gtk_scale_set_digits(GTK_SCALE(val1_bar),2);
    gtk_scale_set_draw_value(GTK_SCALE(val1_bar),TRUE);
    gtk_box_pack_start(GTK_BOX(hbox),label,TRUE,TRUE,0);
    gtk_box_pack_start(GTK_BOX(hbox),val1_bar,TRUE,TRUE,0);
    gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

    hbox = gtk_hbox_new(FALSE,0);
    label = gtk_label_new("out mix 2");
    val2 = (GtkAdjustment *)gtk_adjustment_new(this->out2,0.0,1.0,0.01,0.01,0.0);
    gtk_signal_connect(GTK_OBJECT(val2),"value_changed",GTK_SIGNAL_FUNC(val2_changed),data);
    val2_bar = gtk_hscale_new(val2);
    gtk_scale_set_digits(GTK_SCALE(val2_bar),2);
    gtk_scale_set_draw_value(GTK_SCALE(val2_bar),TRUE);
    gtk_box_pack_start(GTK_BOX(hbox),label,TRUE,TRUE,0);
    gtk_box_pack_start(GTK_BOX(hbox),val2_bar,TRUE,TRUE,0);
    gtk_box_pack_start(GTK_BOX(vbox),hbox,TRUE,TRUE,0);

    
    gtk_widget_show_all(GTK_WIDGET(this->window));
}
static char *get_state(void *data){
    char *state_string;
    pluginData *this = (pluginData *)data;
    state_string = (char *)calloc(sizeof(char),256);
    sprintf(state_string,"%f %f",this->out1,this->out2);
    return state_string;
}

static int set_state(void *data, char *state_string){
    pluginData *this = (pluginData *)data;
    sscanf(state_string,"%f %f",&this->out1,&this->out2);
    return 1;
}

static int process(void *data, short int **smpdata, int size, int sampf){
    pluginData *this = (pluginData *)data;
    int i;
    for(i=0;i<size;i++){
	smpdata[1][i] = smpdata[0][i] * this->out2;
	smpdata[0][i] = smpdata[0][i] * this->out1;
    }
    return 1;
}

Plugin onetotwomix_plugin_data={"1to2 Mix",
    cleanup,
    init,
    configure,
    process,
    get_state,
    set_state,
    1,
    2};

Plugin *get_plugin_info(void){
    return &onetotwomix_plugin_data;
}
