#include <gtk/gtk.h>
#include <plugin.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

/* Ported from stompboxes */


struct plugin_data{
    int countValue;
    short *delayMemory;
    short delayIndex[100000];

    int delayTime;
    int minDelay;
    int depth;

    float inMix;
    float outMix;
    float feedback;

    int rate;
    int counterRate;

    int counterLFO;
    float sinLFO[20];

    GtkWidget *config_window;
};

static void cleanup(void *data){
    struct plugin_data *temp;
    temp=data;
    if(temp->config_window != NULL)
	gtk_widget_destroy(temp->config_window);
    free(temp);
}

static void *init(void){
    struct plugin_data *temp;
    int i;
    double x;
    temp=(struct plugin_data *)malloc(sizeof(struct plugin_data));
    temp->delayMemory = temp->delayIndex;
    temp->countValue = 0;

    /*initial settings*/
    temp->rate = 14000;
    temp->delayTime = 200;
    temp->minDelay = 140;

    temp->depth = temp->delayTime - temp->minDelay;
    temp->counterLFO = 0;
    temp->counterRate = 0;

    temp->inMix = 1.0;
    temp->outMix = 1.0;

    temp->feedback = .1;
    /*initialize delay buffer*/
	for(i=0;i<temp->delayTime;i++)
	    temp->delayMemory[i]=0;
    for(i=0;i<20;i++)
    {
	x = (2*M_PI/20) * i;
	temp->sinLFO[i] = (float)(1 -(1 + cos(x))/2);
    }
    temp->config_window = NULL;
    return temp;
}
static gint rate_changed(GtkWidget *adj, int *data){
   *data = 5000.0 + 250 * GTK_ADJUSTMENT(adj)->value;
   return TRUE;
}
static gint window_destroy(GtkWidget *window, gpointer data){
   struct plugin_data *plug = (struct plugin_data *)data;
   gtk_widget_destroy(window);
   plug->config_window = NULL;
   return TRUE;
}


static void configure(void *data){
    struct plugin_data *plug;
    GtkAdjustment *adj;
    GtkWidget *hbar;
    GtkWidget *label;
    GtkWidget *hbox;
    plug = data;
    if(plug->config_window==NULL){
	plug->config_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(plug->config_window),"Chorus Stompbox");
	gtk_widget_set_usize(GTK_WIDGET(plug->config_window),300,40);
	gtk_signal_connect(GTK_OBJECT(plug->config_window),"destroy",
		GTK_SIGNAL_FUNC(window_destroy),data);
	hbox = gtk_hbox_new(FALSE,0);
	gtk_container_add(GTK_CONTAINER(plug->config_window),hbox);

	label = gtk_label_new("Chorus Rate\n");
	adj = (GtkAdjustment *)gtk_adjustment_new(50,0,100, 0.01, 0.01, 0.0);
	gtk_signal_connect(GTK_OBJECT(adj),"value_changed",
		GTK_SIGNAL_FUNC(rate_changed),&(plug->rate));
	hbar = gtk_hscale_new(GTK_ADJUSTMENT(adj));
	gtk_scale_set_digits(GTK_SCALE(hbar),2);
	gtk_scale_set_draw_value(GTK_SCALE(hbar), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox),label,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(hbox), hbar,TRUE,TRUE,0);
	gtk_widget_show_all(GTK_WIDGET(plug->config_window));
    }

    return;

}

short clip_check(int a, int b)
{
    if((a+b)>32767) {
	//printf("Did a clip check!\n");
	return 32767;
    }
    else if((a+b)<-32767){
	//printf("Did a clip check!\n");
	return -32767;
    }
    else return (short)(a+b);

}


static int process(void *data, short int **smpdata, int size, int sampling_frequency){
    struct plugin_data *temp;
    int i;
    short tempInput;
    short *b= smpdata[0];
    int tempOffset1,tempOffset2;
    temp = data;

    temp->depth = temp->delayTime - temp->minDelay;

    for(i=0;i<size/2 ;i++)
    {
	temp->counterRate++;
	temp->counterRate = temp->counterRate % temp->rate;

	temp->counterLFO = (int)(((float)temp->counterRate/(float)temp->rate) * 20.0);
	tempInput=*b;
	tempOffset1=(temp->countValue - temp->minDelay - (int)((float)temp->depth * temp->sinLFO[temp->counterLFO]));
	if(tempOffset1 < 0) tempOffset1 = temp->delayTime + tempOffset1;

	tempOffset2 = tempOffset1 - 50;
	if(tempOffset2 < 0) tempOffset2 = temp->delayTime + tempOffset2;

	//*b++ = (short)(temp->inMix * tempInput + temp->outMix*(temp->delayMemory[tempOffset1])
	// + temp->delayMemory[tempOffset2]);

	*b++ = clip_check(clip_check((int)temp->inMix * (int)tempInput,(int)temp->outMix*((int)temp->delayMemory[tempOffset1])),
		(int)temp->delayMemory[tempOffset2]);

	temp->delayMemory[temp->countValue] = (short)(tempInput + temp->feedback * temp->delayMemory[temp->countValue]);
	temp->countValue++;

	if(temp->delayTime==temp->countValue){
	    temp->countValue = 0;
	    temp->delayMemory = temp->delayIndex;
	}
    }
    return size;
}
static char *get_state(void *data){
    char *state_string;
    struct plugin_data *plug = (struct plugin_data *)data;
    state_string = (char *)calloc(sizeof(char),256);
    sprintf(state_string,"%d",plug->rate);
    return state_string;
}
static int set_state(void *data, char *state_string){
    struct plugin_data *plug = (struct plugin_data *)data;
    sscanf(state_string,"%d",&plug->rate);
    return 1;
}


Plugin plugin_info = {"Chorus",
    cleanup,
    init,
    configure,
    process,
    get_state,
    set_state,
    1,
    1};

    Plugin *get_plugin_info(void){
	return &plugin_info;
    }
