#include <gtk/gtk.h>
#include <plugin.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

struct plugin_data
{
	GtkWidget *window;
	float gain, clip_vol, master_vol;;
};

void *init(void)
{
	struct plugin_data *temp;
	
	temp=(struct plugin_data *)malloc(sizeof(struct plugin_data));
	temp->gain=100.0;
	temp->clip_vol=100.0;
	temp->master_vol=100.0;
	temp->window=NULL;
	
	return temp;
}

static void cleanup(void *data)
{
	if(((struct plugin_data *)data)->window != NULL)
	    gtk_widget_destroy(((struct plugin_data *)data)->window);
	free(data);
}


static void destroy_confwindow(GtkWidget *wind, gpointer data)
{
	gtk_widget_destroy(wind);
	((struct plugin_data *)data)->window=NULL;
}


static void adjustment_update(GtkWidget *adj, float *data)
{
	*data=GTK_ADJUSTMENT(adj)->value;	
}

static void configure(void *data)
{
	GtkWidget *hscrollbar;
	GtkWidget *label, *hbox;
	GtkWidget *vbox;
	GtkObject *vol_adj;
	struct plugin_data *pdata;

	pdata=data;
	
	if(pdata->window != NULL) return;

	pdata->window=gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_container_set_border_width(GTK_CONTAINER(pdata->window), 10);
	gtk_window_set_title(GTK_WINDOW(pdata->window), "Distortion 2");
	gtk_widget_set_usize(pdata->window, 300, 120);
	gtk_signal_connect(GTK_OBJECT(pdata->window), "destroy", GTK_SIGNAL_FUNC(destroy_confwindow), pdata);
	
	hbox=gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(pdata->window), hbox);
	gtk_widget_show(hbox);

	vbox=gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);
	label=gtk_label_new("Gain (%)");
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 10);
	gtk_widget_show(label);
	label=gtk_label_new("Clip Level (%)");
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 10);
	gtk_widget_show(label);
	label=gtk_label_new("Master Volume (%)");
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 10);
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);



	vbox=gtk_vbox_new(FALSE, 0);
	gtk_widget_show(vbox);
	
	vol_adj=gtk_adjustment_new(pdata->gain, 100.0, 5000.0, 10.0, 10.0, 0.0);
	gtk_signal_connect(vol_adj, "value_changed", GTK_SIGNAL_FUNC(adjustment_update), &(pdata->gain));
	hscrollbar=gtk_hscale_new(GTK_ADJUSTMENT(vol_adj));
	gtk_widget_show(hscrollbar);
	
	gtk_box_pack_start(GTK_BOX(vbox), hscrollbar, TRUE, TRUE, 0);

	vol_adj=gtk_adjustment_new(pdata->clip_vol, 0.0, 100.0, 1.0, 1.0, 0.0);
	gtk_signal_connect(vol_adj, "value_changed", GTK_SIGNAL_FUNC(adjustment_update), &(pdata->clip_vol));
	hscrollbar=gtk_hscale_new(GTK_ADJUSTMENT(vol_adj));
	gtk_widget_show(hscrollbar);
	
	gtk_box_pack_start(GTK_BOX(vbox), hscrollbar, TRUE, TRUE, 0);

	vol_adj=gtk_adjustment_new(pdata->master_vol, 0.0, 500.0, 1.0, 1.0, 0.0);
	gtk_signal_connect(vol_adj, "value_changed", GTK_SIGNAL_FUNC(adjustment_update), &(pdata->master_vol));
	hscrollbar=gtk_hscale_new(GTK_ADJUSTMENT(vol_adj));
	gtk_widget_show(hscrollbar);
	
	gtk_box_pack_start(GTK_BOX(vbox), hscrollbar, TRUE, TRUE, 0);

	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);

	gtk_widget_show(pdata->window);
}


static int process(void *data, short int **smpdata, int size, int sampf)
{
	int i;
	register float temp;
	float gain, clip, volume;
	/*float delta;*/
	struct plugin_data *pdata;
	short int *channel = smpdata[0];

	pdata=data;
	gain=pdata->gain / 100.0;
	clip=pdata->clip_vol * 32768.0 / 100.0;
	volume=pdata->master_vol / 100.0;
	for(i=0; i < (size >> 1); i++)
	{
		temp=(float)channel[i];
		temp=temp * gain;

		if(temp > clip) temp=clip - temp;
		else if(temp < -clip) temp=-clip + temp;
		temp=temp * volume;

		if(temp > 32767.0) temp=32767.0;
		else if(temp < -32768.0) temp=-32768.0;
		channel[i]=(short int)temp;
	}
	return size;
}

static char *get_state(void *data){
    char *state_string;
    struct plugin_data *plug = (struct plugin_data *)data;
    state_string = (char *)calloc(sizeof(char),256);
    sprintf(state_string,"%f %f %f",plug->gain,plug->clip_vol,plug->master_vol);
    return state_string;
}

static int set_state(void *data, char *state_string){
    struct plugin_data *plug = (struct plugin_data *)data;
    sscanf(state_string,"%f %f %f",&plug->gain,&plug->clip_vol,&plug->master_vol);
    return 1;
}



Plugin distortion_plugin_data={"Distortion2",
    cleanup,
    init,
    configure,
    process,
    get_state,
    set_state,
	1,1};

Plugin *get_plugin_info(void)
{
    return &distortion_plugin_data;
}
