#include <gtk/gtk.h>
#include <plugin.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


struct plugin_data_str
{
	GtkWidget *window;
	float threshold;
	float hold_time;
	float release_time; 
	int hold_time_counter;
	int release_time_counter;
	int active;
};

void *init(void)
{
	struct plugin_data_str *temp;
	
	temp=(struct plugin_data_str *)malloc(sizeof(struct plugin_data_str));
	temp->threshold=0.0;
	temp->hold_time=0.5;
	temp->release_time=0.5;	
	temp->hold_time_counter=0;
	temp->window=NULL;
	temp->active=0;
	
	return temp;
}

static void cleanup(void *data)
{
	if(((struct plugin_data_str *)data)->window != NULL)
	    gtk_widget_destroy(((struct plugin_data_str *)data)->window);
	free(data);
}


static void destroy_confwindow(GtkWidget *wind, gpointer data)
{
	gtk_widget_destroy(wind);
	((struct plugin_data_str *)data)->window=NULL;
}


static void adjustment_update(GtkWidget *adj, float *data)
{
	*data=GTK_ADJUSTMENT(adj)->value;	
}

static void configure(void *data)
{
	GtkWidget *hscrollbar;
	GtkObject *vol_adj;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *label;
	struct plugin_data_str *pdata=data;
	
	if(pdata->window != NULL) return;

	pdata->window=gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(pdata->window), "Noise Gate Plugin");
	gtk_container_set_border_width(GTK_CONTAINER(pdata->window), 10);
	gtk_widget_set_usize(pdata->window, 300, 200);
	
	vbox=gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(pdata->window), vbox);

	hbox=gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox);

	label=gtk_label_new("Threshold (%)");
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);

	vol_adj=gtk_adjustment_new(pdata->threshold, 0.0, 100.0, 1.0, 1.0, 0.0);
	gtk_signal_connect(GTK_OBJECT(vol_adj), "value_changed", GTK_SIGNAL_FUNC (adjustment_update), &(pdata->threshold));

	hscrollbar=gtk_hscale_new(GTK_ADJUSTMENT(vol_adj));
	gtk_scale_set_draw_value(GTK_SCALE(hscrollbar), TRUE);
	gtk_scale_set_digits(GTK_SCALE(hscrollbar), 0);
	gtk_box_pack_start(GTK_BOX(hbox), hscrollbar, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
	gtk_widget_show(hscrollbar);	
	
	
	
	hbox=gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox);

	label=gtk_label_new("Hold time (s)");
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);

	vol_adj=gtk_adjustment_new(pdata->hold_time, 0.001, 1.0, 0.001, 0.001, 0.0);
	gtk_signal_connect(GTK_OBJECT(vol_adj), "value_changed",GTK_SIGNAL_FUNC (adjustment_update), &(pdata->hold_time));

	hscrollbar=gtk_hscale_new(GTK_ADJUSTMENT(vol_adj));
	gtk_scale_set_draw_value(GTK_SCALE(hscrollbar), TRUE);
	gtk_scale_set_digits(GTK_SCALE(hscrollbar), 3);
	gtk_box_pack_start(GTK_BOX(hbox), hscrollbar, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);	
	gtk_widget_show(hscrollbar);	
	
	
	hbox=gtk_hbox_new(FALSE, 0);
	gtk_widget_show(hbox);
	
	label=gtk_label_new("Release time (s)");
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);

	vol_adj=gtk_adjustment_new(pdata->release_time, 0.001, 1.0, 0.001, 0.001, 0.0);
	gtk_signal_connect(GTK_OBJECT(vol_adj), "value_changed",GTK_SIGNAL_FUNC (adjustment_update), &(pdata->release_time));

	hscrollbar=gtk_hscale_new(GTK_ADJUSTMENT(vol_adj));
	gtk_scale_set_draw_value(GTK_SCALE(hscrollbar), TRUE);
	gtk_scale_set_digits(GTK_SCALE(hscrollbar), 3);
	gtk_box_pack_start(GTK_BOX(hbox), hscrollbar, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);	
	gtk_widget_show(hscrollbar);		

	gtk_signal_connect(GTK_OBJECT(pdata->window), "destroy", GTK_SIGNAL_FUNC(destroy_confwindow), data);

	gtk_widget_show(vbox);
	gtk_widget_show(pdata->window);
}


static int process(void *data, short int **smpdata, int size, int sampf)
{
	int i;
	float threshold_value;
	register int threshold_value_int;
	struct plugin_data_str *pdata=data;
	int temp/*, temp1*/;
	float gate_volume;
	int release_time_samples;
	short int *channel = smpdata[0];

	threshold_value=(((struct plugin_data_str *)data)->threshold) / 100.0;
	threshold_value_int=(int)(threshold_value * 32767.0);
	for(i=0; i < (size >> 1); i++)
	{
		temp=channel[i];
		if((temp < threshold_value_int) && (temp > -threshold_value_int))
		{
		    if(pdata->hold_time_counter >= (int)(pdata->hold_time * sampf))
		    {
			if(pdata->active == 0)
			{
			    pdata->active=1;
			    pdata->release_time_counter=0;
			}			
			else
			{
			    release_time_samples=(int)(pdata->release_time * (float)sampf);

			    if(pdata->release_time_counter < release_time_samples) pdata->release_time_counter++;
			    else pdata->release_time_counter=release_time_samples;

			    gate_volume=(1.0 - ((float)(pdata->release_time_counter) / (float)release_time_samples));

			    temp*=gate_volume;
			}
		    }
		    else
		    {
			pdata->hold_time_counter++;
		    }
		}
		else
		{
		    pdata->hold_time_counter=0;
		    pdata->active=0;
		}				
		
		channel[i]=(short int)(temp);
	}
	return size;
}
static char *get_state(void *data){
    char *state_string;
    struct plugin_data_str *plug = (struct plugin_data_str *)data;
    state_string = (char *)calloc(sizeof(char),256);
    sprintf(state_string,"%f %f %f",plug->threshold,plug->hold_time,plug->release_time);
    return  state_string;
}

static int set_state(void *data, char *state_string){
    struct plugin_data_str *plug = (struct plugin_data_str *)data;
    sscanf(state_string,"%f %f %f",&plug->threshold,&plug->hold_time,&plug->release_time);
    return 1;
}




Plugin volume_plugin_data={"NoiseGate",
cleanup,
init,
configure,
process,
get_state,
set_state,
1,1};

Plugin *get_plugin_info(void)
{
	return &volume_plugin_data;
}
