#include <gtk/gtk.h>
#include <plugin.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

struct plugin_data{
    GtkWidget *config_window;
    int counter;
    float time;
    short int *back_data;
};

static void cleanup(void *data)
{
	struct plugin_data *temp;

	temp=data;
	if(temp->config_window != NULL)
		gtk_widget_destroy(temp->config_window);
	if(temp->back_data != NULL) free(temp->back_data);
	free(temp);
}
static void *init(void)
{
	struct plugin_data *temp;

	temp=(struct plugin_data *)malloc(sizeof(struct plugin_data));
	temp->back_data = (short *)calloc(sizeof(short) * 44100 * 6,1);
	temp->time = 1.0;
	temp->counter = 0;
	temp->config_window = NULL;
	return temp;
}

static void time_changed(GtkWidget *adj, gpointer data){
    struct plugin_data *temp;
    temp = data;
    temp->time = GTK_ADJUSTMENT(adj)->value;
}

static void config_window_destroy(GtkWidget *w, gpointer data)
{
	struct plugin_data *temp;

	temp=data;

	if(temp->config_window != NULL)
		gtk_widget_destroy(temp->config_window);
	
	temp->config_window=NULL;
}


static void configure(void *data)
{
    struct plugin_data *temp;
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *label;
    GtkObject *time_adj;
    GtkWidget *time_bar;

    temp=data;
    if(temp->config_window != NULL) return;

    temp->config_window = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_title(GTK_WINDOW(temp->config_window), "Reverse Plugin");
    gtk_widget_set_usize(GTK_WIDGET(temp->config_window), 300,40);
    gtk_signal_connect (GTK_OBJECT(temp->config_window),"destroy",
	    		GTK_SIGNAL_FUNC(config_window_destroy), data);
    vbox = gtk_vbox_new(FALSE,0);
    gtk_container_add(GTK_CONTAINER(temp->config_window),vbox);

    hbox = gtk_hbox_new(FALSE,0);
    
    label = gtk_label_new("Time (seconds)");
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE,0);
    time_adj = gtk_adjustment_new(temp->time, 0.1, 3, 0.01, 0.01, 0.0);
    gtk_signal_connect(GTK_OBJECT(time_adj),"value_changed",
	    		GTK_SIGNAL_FUNC(time_changed),data);
    time_bar = gtk_hscale_new(GTK_ADJUSTMENT(time_adj));
    gtk_scale_set_digits(GTK_SCALE(time_bar),2);
    gtk_scale_set_draw_value(GTK_SCALE(time_bar), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), time_bar, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
    gtk_widget_show_all(GTK_WIDGET(temp->config_window));
}

static int process(void *data, short int **smpdata, int size, int sampf)
{
    struct plugin_data *temp = data;
    short *b = smpdata[0],temp_input;
    int i;
    int sampling_frequency = sampf;
    for(i=0;i<size/2;i++){
	temp_input = *b;
	if(temp->counter == (int) ((sampling_frequency * 2.0 * temp->time)/2))
	{
	    *b++ = temp_input;
	    temp->back_data[temp->counter] = temp_input;
	}
	else if(temp->counter < (int) (sampling_frequency * 2.0 * temp->time)){
	    *b++ = temp->back_data[(int)(sampling_frequency * 2.0 * temp->time) - temp->counter];
	    temp->back_data[temp->counter] = temp_input;
	}
	else {
	    temp->counter = -1;
	}
	temp->counter++;
    }
    return 1;
}

static char *get_state(void *data){
    char *state_string;
    struct plugin_data *plug = (struct plugin_data *)data;
    state_string = (char *)calloc(sizeof(char),256);
    sprintf(state_string,"%f",plug->time);
    return state_string;
}

static int set_state(void *data, char *state_string){
    struct plugin_data *plug = (struct plugin_data *)data;
    sscanf(state_string,"%f",&(plug->time));
    return 1;
}


Plugin plugin_info={"Reverse",
    	cleanup,
	init,
	configure,
	process,
	get_state,
	set_state,
	1,
	1};

Plugin *get_plugin_info(void)
{
    return &plugin_info;
}


