#include <gtk/gtk.h>
#include <stdlib.h>
#include <plugin.h>
#include <stdio.h>

typedef struct{
   gfloat vol; 
   GtkWidget *window;
}pluginData;

void *init(void){
   pluginData *data;
   data  = (pluginData *)malloc(sizeof(pluginData));
   data->vol = 1.0;
   data->window = NULL;
   return data;
}

static void cleanup(void *data){
    pluginData *plug = (pluginData *)data;
    if(plug->window!=NULL) gtk_widget_destroy(plug->window);
    free(data);
}

static gint window_destroy(GtkWidget *window, gpointer data){
   pluginData *plug = (pluginData *)data;
   gtk_widget_destroy(window);
   plug->window = NULL;
   return TRUE;
}

static gint vol_changed(GtkWidget *widget, gpointer data){
    pluginData *plug = (pluginData *)data;
    plug->vol = GTK_ADJUSTMENT(widget)->value;
    return TRUE;
}

static void configure(void *data){
   GtkAdjustment *adj;
   GtkWidget *hbar;
   GtkWidget *label;
   GtkWidget *hbox;
   pluginData *plug = (pluginData *)data;
   if(plug->window==NULL){
       plug->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
       gtk_window_set_title(GTK_WINDOW(plug->window),"Volume Stompbox");
       gtk_widget_set_usize(GTK_WIDGET(plug->window),300,40);
       gtk_signal_connect(GTK_OBJECT(plug->window),"destroy",
	       GTK_SIGNAL_FUNC(window_destroy),data);
       hbox = gtk_hbox_new(FALSE,0);
       gtk_container_add(GTK_CONTAINER(plug->window),hbox);

       label = gtk_label_new("Volume factor\n");
       adj = (GtkAdjustment *)gtk_adjustment_new(plug->vol,0,3, 0.01, 0.01, 0.0);
       gtk_signal_connect(GTK_OBJECT(adj),"value_changed",
	       GTK_SIGNAL_FUNC(vol_changed),data);
       hbar = gtk_hscale_new(GTK_ADJUSTMENT(adj));
       gtk_scale_set_digits(GTK_SCALE(hbar),2);
       gtk_scale_set_draw_value(GTK_SCALE(hbar), TRUE);
       gtk_box_pack_start(GTK_BOX(hbox),label,TRUE,TRUE,0);
       gtk_box_pack_start(GTK_BOX(hbox), hbar,TRUE,TRUE,0);
       gtk_widget_show_all(GTK_WIDGET(plug->window));
   }
}

static int process(void *data, short int **smpdata, int size, int sampf){
    short int *channel;
    int i;
    pluginData *plug = (pluginData *)data;
    channel = smpdata[0];
    for(i=0;i<size;i++){
	channel[i] = (short int)(channel[i] * plug->vol);
    }
    return 1;
}

static char *get_state(void *data){
    char *state_string;
    pluginData *plug = (pluginData *)data;
    state_string = (char *)calloc(sizeof(char),256);
    sprintf(state_string,"%f",(float)plug->vol);
    return state_string;
}

static int set_state(void *data, char *state_string){
    pluginData *plug = (pluginData *)data;
    float vol;
    sscanf(state_string,"%f",&vol);
    plug->vol = vol;
    return 1;
}

Plugin volume_plugin_data={"Volume",
    cleanup,
    init,
    configure,
    process,
    get_state,
    set_state,
    1,
    1};

Plugin *get_plugin_info(void){
    return &volume_plugin_data;
}

